/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import java.io.*;
import java.lang.*;
import java.awt.Component;
import java.util.Stack;
import javax.swing.JOptionPane;

/**
 * A class for reporting SpiTran messages using any combination of a modal
 * dialog, a log file, and stdout printing.
 *
 * @author Anish Malhotra, Cadence Design Systems
 * @see s2iDBOpenListener
 */
public class s2iError
{
   /**
    * Show the message in a popup modal dialog.
    */
   public static final int POPUP   = 0x00001;
   /**
    * Show the message the SpiTran.log file.
    */
   public static final int LOG     = 0x00002;
   /**
    * Show the message in stdout.
    */
   public static final int DISPLAY = 0x00004;

   /**
    * This is a debug message, only shown in debug mode.
    */
   public static final int DEBUG = 1;
   /**
    * This is a question message, requiring a response.
    */
   public static final int QUEST = 2;
   /**
    * This is an information message.
    */
   public static final int INFO  = 3;
   /**
    * This is a warning message.
    */
   public static final int WARN  = 4;
   /**
    * This is an error message.
    */
   public static final int ERR   = 5;
   /**
    * This is a fatal error message. The application will quit.
    */
   public static final int FATAL = 6;
   /**
    * The highest severity code allowed.
    */
   public static final int N_SEVERITIES = 6;
   /**
    * Vector of message prefix strings for each severity level.
    */
   private static final String[] SEV_STR;
   /**
    * Name of the log file.
    */
   public static final String LogFileName = "SpiTran.log";
   /**
    * The log file.
    */
   private static PrintWriter logFile;
   /**
    * Title for popup dialog windows.
    */
   private static final String DLG_TITLE = "SpiTran";
   /**
    * Number of log file versions to archive.
    */
   private static final int MAXLOGS = 3;
   /**
    * Is the log file open?
    */
   private static boolean logOpen = false;
   /**
    * Show DEBUG messages?
    */
   private static final boolean DBG_MSG ;


   //An initializer block to construct the array of severity Strings.
   static
   {
      SEV_STR = new String[N_SEVERITIES + 1];
      SEV_STR[DEBUG] = new String("D- ");
      SEV_STR[QUEST] = new String("");
      SEV_STR[INFO] = new String("");
      SEV_STR[WARN] = new String("W- ");
      SEV_STR[ERR] = new String("E- ");
      SEV_STR[FATAL] = new String("");
   }
   //An initializer block to initialize static variables
   static
   {
      String DEBUG_val = System.getProperty("DEBUG");
      DBG_MSG = (DEBUG_val == null) ? false : true;
   }

   /**
    * Show a debug message. This is printed to the log and stdout if in
    * DEBUG mode.
    * @param Msg The debug message String
    */
   public static final void Debug(String msg)
   {
      PrintMsg(msg, DEBUG, LOG | DISPLAY, null);
   }

   /**
    * Show a message using all specified means.
    * @param Msg The message String
    * @param The severity level, one of: DEBUG QUEST INFO WARN ERR FATAL
    * @param Options Any combination of: POPUP LOG DISPLAY
    * @param parent Determines the Frame in which the dialog is displayed.
    *               If null, or if the parent Component has no Frame, a
    *               default Frame is used.
    */
   public static final void PrintMsg(String Msg, int severity,
          int Options, Component parent)
   {
      if ((Options & POPUP) != 0)
      {
         PrintDlgMsg(Msg, severity, parent);
      }

      if ((Options & LOG) != 0)
      {
         PrintLogMsg(SEV_STR[severity] + Msg, severity);
      }

      if ((Options & DISPLAY) != 0)
      {
         if ((severity!=DEBUG) || (DBG_MSG))
            System.out.println(SEV_STR[severity] + Msg);
      }
   }

   /**
    * Show a message in a dialog box.
    * @param Msg The message String
    * @param The severity level, one of: DEBUG QUEST INFO WARN ERR FATAL
    * @param parent Determines the Frame in which the dialog is displayed.
    *               If null, or if the parent Component has no Frame, a
    *               default Frame is used.
    */
   private static final void PrintDlgMsg(String Msg, int severity,
           Component parent)
   {
         int msgType = JOptionPane.PLAIN_MESSAGE;

         switch (severity)
         {
            case DEBUG: msgType = JOptionPane.PLAIN_MESSAGE;
                        break;
            case QUEST: msgType = JOptionPane.QUESTION_MESSAGE;;
                        break;
            case INFO:  msgType = JOptionPane.INFORMATION_MESSAGE;
                        break;
            case WARN:  msgType = JOptionPane.WARNING_MESSAGE;
                        break;
            case ERR:   msgType = JOptionPane.ERROR_MESSAGE;
                        break;
            case FATAL: msgType = JOptionPane.ERROR_MESSAGE;
                        break;
         }

         if ((severity!=DEBUG) || (DBG_MSG))
            JOptionPane.showMessageDialog(parent, Msg, DLG_TITLE, msgType);
   }

   /**
    * Show a message by printing to stdout.
    * @param Msg The message String
    * @param The severity level, one of: DEBUG QUEST INFO WARN ERR FATAL
    */
   private static final void PrintLogMsg(String Msg, int severity)
   {
      if (IsLogOpen())
      {
         if ((severity!=DEBUG) || (DBG_MSG))
            logFile.println(Msg);
      }
   }

   /** Does the following conversion:
    * SpiTran.log --> SpiTran.log,1 --> SpiTran.log,2 --> SpiTran.log,3
    * @param Name Name String of the current log file.
    * @return Name String of the next log file.
    */
   private static String nxtLogName(String Name)
   {
      String newName;
      char  endChar;
      int len, endDig;

      len = Name.length();
      endChar = Name.toCharArray()[len-1]; 
      if (Character.isDigit(endChar))
      {
         len = len -2;  //because the last two characters are to be overwritten
         endDig = Integer.parseInt((new Character(endChar)).toString());
         endDig++;
      }
      else
         endDig = 1;

      if (endDig > MAXLOGS) 
         return (null);
      // Make the new String
      newName = new String(Name.substring(0, len) + "," + new Integer(endDig));
      return(newName);
   }
      
/**
 * Archive previous log files by renaming them with number suffixes.
 */
   private static final void mvLogs()
   {
      File log;
      File nextLog, source, dest = null; 
      Stack files = new Stack();
      String nextName, NameWPath;

      if (s2iGlobals.ProjDirFile != null)
      {
         /* Check if the project directory is writable. If not, give the message
          * about the log file not being generated.
          */
         NameWPath = new String(s2iGlobals.ProjDirFile.toString() +
                                java.io.File.separator + LogFileName);
      }
      else
      {
        /* Check the pwd to be writable */
        NameWPath = LogFileName;
      }
      log = new File(NameWPath);

      if (log.exists())
      {
         files.push(log);
         while(log.exists())
         {
            nextName = nxtLogName(log.toString());
            if (nextName == null)
            {
               files.push(null);
               break;
            }
            nextLog = new File(nextName);
            files.push(nextLog);
            log = nextLog;
         }

         // Now move the files
         if (!files.empty()) dest = (File) files.pop();
         while( !files.empty())
         {
            source = (File) files.pop();
            if (dest != null)
               source.renameTo(dest);
            dest = source;
         }
      } // If log files existed
   }


/**
 * Open an existing Log File and appends to it.
 * Not Yet implemented - hence private.
 */
   private static final boolean openLog()
   {
      return(false);
   }

/**
 * Open the log file. This archives previous log files and starts a new one.
 * @return true if the file was opened
 * @exception java.io.IOException
 */
   public static final boolean openNewLog()
      throws java.io.IOException
   {
      String fullName;

      mvLogs();
      if (s2iGlobals.ProjDirFile != null)
      {
         fullName = new String (s2iGlobals.ProjDirFile.toString() +
                               java.io.File.separator + LogFileName);
      }
      else
      {
         fullName = LogFileName;
      }
      //Create a FileWriter in non-append mode; a PrintWriter with auto-flush
      logFile = new PrintWriter( new BufferedWriter
                (new FileWriter(fullName, false)), true);
      logOpen = (logFile != null);
      return(logOpen);

   }

/**
 * Close the log file.
 */
   public static final void closeLog()
   {
      logFile.close();
      logOpen = false;
   }
/**
 * Determine if the log file is open.
 * @return true if the log file is currently open.
 */
   public static final boolean IsLogOpen()
   {
      return(logOpen);
   }

/**
 * Test the s2iError class.
 * @param args unused
 */
    public static void main(String args[])
    {
       boolean OK_open = false;

       try{
          OK_open = openNewLog();
       }
       catch (java.io.IOException ex)
       {
          System.out.println("failure to open the Log-File");
       }
       if (!OK_open)
         System.out.println("failure to open the Log-File");
       else
       {
          if (!IsLogOpen())
            System.out.println("Mismatch in determining opened log");
       }

       PrintMsg("This is a dummy warning", WARN, LOG | POPUP | DISPLAY, null); 
       PrintMsg("Only in the Log ", INFO, LOG , null); 
       PrintMsg("Only on the Popup ", QUEST, POPUP , null); 
       PrintMsg("This is a dummy error", ERR, LOG | POPUP | DISPLAY, null); 
       
       if (IsLogOpen()) 
          closeLog();
    }
}
