/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */

import java.io.*;
import javax.swing.JTextArea;

/**
 * A class for running programs with a text window to show the output.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */
public class s2iProcess implements Runnable{
/**
 * The command strings to run
 */
   private String[] cmd;
/**
 * The text area to display output in
 */
   private JTextArea textArea = null;

/**
 * Construct a process with a series of commands, discarding output
 * @param arg The Vector of command Strings
 */
   public s2iProcess(String arg[])
   {
      // Copy the command Strings
      cmd = new String[arg.length];
      int i = 0;
      for (i=0; (i < arg.length); i++)
      {
         cmd[i] = new String(arg[i]);
      }
   }

/**
 * Construct a process with a series of commands, displaying output in a text area
 * @param arg The Vector of command Strings
 */
   public s2iProcess(String arg[], JTextArea ta)
   {
      cmd = new String[arg.length];
      int i = 0;
      for (i=0; (i < arg.length); i++)
      {
         cmd[i] = new String(arg[i]);
      }

      textArea = ta;
   }

/**
 * Run the commands. This works only if a text area has been specified.
 * Error messages are printed to stdout. This method waits for the process.
 */
   public void run()
   {
      try{
         if (textArea != null)
         {
            for (int i=0; (i < cmd.length); i++)
            {
               Process child = runProgInTxtArea(cmd[i], textArea);
               if (child.waitFor()!=0)
               {
                  child.exitValue();
                  return;
               }
            }
         }
      }
      catch (java.io.IOException ioe)
      {
         s2iError.PrintMsg(ioe.getMessage(), s2iError.ERR,
            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY, null);
         s2iError.PrintMsg("Program Aborted", s2iError.FATAL,
            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY, null);
      }
      catch (InterruptedException intrpt)
      {
         System.out.println();
         s2iError.PrintMsg("Execution of the s2iProcess program interrupted",
            s2iError.ERR,
            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY, null);
         s2iError.PrintMsg("Program Aborted", s2iError.FATAL,
            s2iError.POPUP | s2iError.LOG | s2iError.DISPLAY, null);
      }

   }

/**
 * Run a single command without waiting. Input and out go to the console.
 * @param cmd The command String to run
 * @return The Process started up
 */
   public static Process runProg(String cmd)
      throws java.io.IOException
   {
      Process proc = Runtime.getRuntime().exec(cmd);
      //plugTogether(System.in, proc.getOutputStream());
      plugTogether(System.out, proc.getInputStream());
      plugTogether(System.err, proc.getErrorStream());
      return proc;
   }

/**
 * Run a single command without waiting. Input and out go to the test area.
 * @param cmd The command String to run
 * @param tArea The text area to display output in
 * @return The Process started up
 */
   public static Process runProgInTxtArea(String cmd, JTextArea tArea)
      throws java.io.IOException
   {
      Process proc = Runtime.getRuntime().exec(cmd);
      //plugTogether(System.in, proc.getOutputStream());
      plugTogether(tArea, proc.getInputStream());
      plugTogether(tArea, proc.getErrorStream());
      return proc;
   }

/**
 * test the class. Each argument must be a whole command. For example:
 * <PRE>
 * java s2iProcess "write s2iProcess.java"
 * </PRE>
 Runs an editor on this java file.
 */
   public static void main(String argv[])
   {
      for (int i = 0; i < argv.length; ++i)
      {
        String cmd = argv[i];
        try
        {
           Process child = runProg(cmd);
           if (child.waitFor() != 0)
             System.exit(child.exitValue());
           else
             System.out.println("Program " + cmd + " executed sucessfully");
        }
        catch (java.io.IOException ioe)
        {
          System.out.println(ioe.getMessage());
          System.out.println("Program " + cmd + " aborted");
        }
        catch (InterruptedException intrpt)
        {
          System.out.println(intrpt.getMessage());
          System.out.println("Program " + cmd + " interrupted");
        }
      }
    }

/**
 * Arrange to have all input from a stream printed in a text area. This is
 * done using a new thread.
 * @param textarea The text area that will display output
 * @param in The input stream that is to be printed
 */
   private static void plugTogether(JTextArea textarea, InputStream in)
   {
      s2iIOPlug p = new s2iIOPlug(textarea, in);
      Thread plugThread = new Thread(p);
      plugThread.start();
   }

/**
 * Arrange to have all input from a stream copied to another stream. This is
 * done using a new thread.
 * @param in The input stream to copy from
 * @param out The output stream to copy to
 */
   private static void plugTogether(InputStream in, OutputStream out)
   {
      s2iIOPlug p = new s2iIOPlug(in, out);
      Thread plugThread = new Thread(p);
      plugThread.start();
   }

/**
 * Arrange to have all input from a stream copied to another stream. This is
 * done using a new thread.
 * @param out The output stream to copy to
 * @param in The input stream to copy from
 */
   private static void plugTogether(OutputStream out, InputStream in)
   {
      Thread plugThread = new Thread(new s2iIOPlug(in, out));
      plugThread.start();
   }
}

/**
 * A class for copying stream input to another stream or to a textarea.
 *
 * @author Anish Malhotra, Cadence Design Systems
 */
class s2iIOPlug implements Runnable
{
/**
 * The input stream to read from, buffered for efficiency
 */
   private BufferedReader in;
/**
 * The output stream to print to (may be null)
 */
   private OutputStream out = null;
/**
 * The text area to print to (may be null)
 */
   JTextArea txtArea = null;

/**
 * Construct a plug that connects two streams
 * @param in The input stream to copy from
 * @param out The output stream to copy to
 */
   public s2iIOPlug(InputStream in, OutputStream out)
   {
      this.in =new BufferedReader(new InputStreamReader(in));
      this.out = out;
   }

/**
 * Construct a plug that connects an input stream to a text area
 * @param ta The text area to copy to
 * @param in The input stream to copy from
 */
   public s2iIOPlug(JTextArea ta, InputStream in)
   {
      this.txtArea = ta;
      this.in =new BufferedReader(new InputStreamReader(in));
   }

/**
 * Copy bytes from an input stream to an output stream. This returns only
 * when there is no more input.
 */
   private void run1()
   {
     try{
     String line;
     System.out.println("From run - 1");
     while((line = in.readLine()) != null)
     {
        out.write(line.getBytes());
        out.write(System.getProperty("line.separator").getBytes());
     }
     }
     catch (java.io.IOException ioe)
     {
        System.out.println(ioe.getMessage());
     }
   }

/**
 * Copy bytes from an input stream to a text area. This returns only
 * when there is no more input.
 */
   private void run2()
   {
     try{
       String line;
       String linesep = System.getProperty("line.separator");
       while((line = in.readLine()) != null)
       {
         txtArea.append(line);
         txtArea.append(linesep);
         int endPos = txtArea.getDocument().getEndPosition().getOffset();
         txtArea.setCaretPosition(endPos-1);
         //java.lang.Thread.currentThread().yield();
       }
     }
     catch (java.io.IOException ioe)
     {
        System.out.println(ioe.getMessage());
     }
   }

/**
 * Decide if output is to a text area or to a stream, and call the appropriate
 * run method.. This returns only when there is no more input.
 */
   public void run()
   {
      if (out !=null)
        run1();
      else if (txtArea != null)
        run2();
   }
}

