/*
 *
 * Copyright 1999, Cadence Design Systems, Incorporated.
 *
 * Cadence Design Systems (Cadence) and the author hereby disclaim
 * all implied warranties, including the implied warranties of merchant-
 * ability and fitness for a particular purpose. Cadence is not liable for
 * any damages incurred by the use of this software and/or documentation,
 * including direct, indirect, special, incidental, or consequential
 * damages.
 *
 * Cadence grants the bearer the right to modify, copy, and redistribute
 * this software and documentation, but requests that Cadence be acknowledged
 * for its contribution in the resulting product.
 *
 */



import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

/** Rather than parsing the SPICE file again and again while looking for
    data, this class reads in the SPICE file once and stores all the data in
    various fields which can be retrieved later through the methods provided. 
    One-time reading of SPICE file has an exception for Include files, Sub-
    ckts, and Sub-ckt definitions which have already been implemented in 
    s2iSpiceReader.
*/

public class s2iSpiceData {

  private String fileNameWPath = null;
  private Date readStartDate; //assign a Date(currentTimeMillis()) 
  private Date readEndDate;   //assign a Date(currentTimeMillis()) 
  private int fileSize;
  private int nLine, nSpiceLine, nChar;

  /* Fields for storing the relevant SPICE file data
    =================================================
  */
  Hashtable voltSrcElems;
  Hashtable currSrcElems;
  Hashtable resistorElems;
  Hashtable subCktElems;
  Hashtable XSubCkts;
  Hashtable nodes;

  /* Some constants
    ===============
  */
  private static final int NONE = 0;
  private static final int TWO_NODES = 1;
  private static final int ALL_BUT_1 = 2;


  public s2iSpiceData()
  {
     voltSrcElems = new Hashtable();
     currSrcElems = new Hashtable();
     resistorElems = new Hashtable();
     subCktElems = new Hashtable();
     XSubCkts = new Hashtable();
     nodes = new Hashtable();
  }

  public s2iSpiceData(String fileNameWPath)
  {
     this();
     this.fileNameWPath = fileNameWPath; 
  }


  public void fillData(String fileNameWPath)
     throws java.io.FileNotFoundException, java.io.IOException
  {
     /*********************************************************************
      * This reads the specified SPICE file and adds the new data to the  *
      * present data. i.e.. the previous data will not be erased but will *
      * will be preserved and added to.                                   *
      ********************************************************************/
     this.fileNameWPath = fileNameWPath;
     fillData();
  }


  public void fillData() 
     throws java.io.FileNotFoundException, java.io.IOException
  {
     if (fileNameWPath == null) return;
     //Open the file for reading
     s2iSpiceReader fileReader = new s2iSpiceReader(fileNameWPath);

     //record the read start time
     readStartDate = new Date(System.currentTimeMillis());

     // read the file and store the relevant data
     Vector dummy = new Vector();
     StringBuffer SpiceLine = new StringBuffer();
     char fstChar;
     StringTokenizer tokens;
     String word;
     int numNodes;

     while( fileReader.getSpiceLine(SpiceLine, dummy) != 0)
     {
       tokens = new StringTokenizer(SpiceLine.toString()); 
       if (tokens.hasMoreTokens())
       {
         numNodes = NONE; 
         word = tokens.nextToken();
         fstChar = word.charAt(0);
         
         if ((fstChar=='v')||(fstChar=='V'))
         {
            voltSrcElems.put(word, word);
            numNodes = TWO_NODES;
         }
         else if ((fstChar=='r')||(fstChar=='R'))
         {
            resistorElems.put(word, word);
            numNodes = TWO_NODES;
         }
         else if ((fstChar=='i')||(fstChar=='I'))
         {
            currSrcElems.put(word, word);
            numNodes = TWO_NODES;
         }
         else if ((fstChar=='x')||(fstChar=='X'))
         {
            subCktElems.put(word, word);
            numNodes = ALL_BUT_1;
         }
         
         switch (numNodes){
              case NONE :
                      break;

              case TWO_NODES :
                      /* In all the three cases above, the element names are 
                         followed by two node names. */
                      int ctr=0;
                      while ((tokens.hasMoreTokens()) && (ctr < 2))
                      {
                         word = tokens.nextToken();
                         nodes.put(word, word);
                         ctr++;
                      }
                      break;

              case ALL_BUT_1 :
                      /* In the case of circuit instantiations, the last
                         word is the name of the instantiated sub-circuit.
                      */
                      word = tokens.nextToken();
                      while (tokens.hasMoreTokens())
                      {
                         nodes.put(word, word);
                         word = tokens.nextToken();
                      }
                      /* The last word */
                      XSubCkts.put(word, word);
                      break;
       } //Switch
       }
     }

     // upon completion of read, record the read end time
     readEndDate = new Date(System.currentTimeMillis());
  }


  public boolean isVoltElem(String elem)
  {
     return voltSrcElems.containsKey(elem);
  }


  public boolean isCurrElem(String elem)
  {
     return currSrcElems.containsKey(elem);
  }


  public boolean isResElem(String elem)
  {
     return resistorElems.containsKey(elem);
  }


  public boolean isSubCktElem(String elem)
  {
     return subCktElems.containsKey(elem);
  }


  public boolean isSubCktInstantiated(String elem)
  {
     return XSubCkts.containsKey(elem);
  }


  public boolean isNode(String elem)
  {
     return nodes.containsKey(elem);
  }

  public Enumeration getVoltElem()
  {
     return voltSrcElems.elements();
  }


  public Enumeration getCurrElem()
  {
     return currSrcElems.elements();
  }


  public Enumeration getResElem()
  {
     return resistorElems.elements();
  }


  public Enumeration getSubCktElem()
  {
     return subCktElems.elements();
  }


  public Enumeration getSubCktInstantiated()
  {
     return XSubCkts.elements();
  }


  public Enumeration getNode()
  {
     return nodes.elements();
  }

  public void addVoltElem(String elem)
  {
     voltSrcElems.put(elem, elem);
  }


  public void addCurrElem(String elem)
  {
     currSrcElems.put(elem, elem);
  }


  public void addResElem(String elem)
  {
     resistorElems.put(elem, elem);
  }


  public void addSubCktElem(String elem)
  {
     subCktElems.put(elem, elem);
  }


  public void addSubCktInstantiated(String elem)
  {
     XSubCkts.put(elem, elem);
  }


  public void addNode(String elem)
  {
     nodes.put(elem, elem);
  }


  public String getUniqueNodeName()
  {
     int ctr = 0;
     String nodeName = s2iGlobals.baseNodeName + "_" + (new String(new Integer(ctr++).toString()));
     while (isNode(nodeName))
     {
        nodeName = s2iGlobals.baseNodeName + "_" + (new String(new Integer(ctr++).toString()));
     }
     
     return(nodeName);
  }


  public String getUniqueResElemName()
  {
     int ctr = 0;
     String resElemName = s2iGlobals.baseRElemName + "_" + (new String(new Integer(ctr++).toString()));
     while (isResElem(resElemName))
     {
        resElemName = s2iGlobals.baseRElemName + "_" + (new String(new Integer(ctr++).toString()));
     }
     
     return(resElemName);
  }


  public String getUniqueVoltElemName()
  {
     int ctr = 0;
     String voltElemName = s2iGlobals.baseVElemName + "_" + (new String(new Integer(ctr++).toString()));
     while (isVoltElem(voltElemName))
     {
        voltElemName = s2iGlobals.baseVElemName + "_" + (new String(new Integer(ctr++).toString()));
     }
     
     return(voltElemName);
  }


  /**************************************************************************
   * This is the main() function that is primarily for sanity checking      *
   *************************************************************************/
 public static void main(String args[])
 {
    System.out.println("No. of Arguments = " + args.length);
    if (args.length != 1)
    {
       System.out.println("Usage : java s2iSpiceData <fileName>");
       System.exit(0);
    }
    if (args[0] == null)
       System.out.println("A null string");
    if (args[0].trim().equals(""))
       System.out.println("A blank string for the fileName") ; 

   /*>>>>>>>>--------Cut Here for Sample data---------<<<<<<<<<
    VvoltElem1 node1 0 5V
    Relem1 my_node1 node1 1K

    VvoltElem2 node3 0 3.3V
    Relem2 my_node2 node3 11K

    Xckt1 node_10 node_11 SpiNode_3 node_13 cktName1

    VvoltElem3 node7 0 3.3V
    Relem3 my_node3 node7 21K

    Xckt3 node_20 SpiNode_0 node_22 SpiNode_1 node_24 node_25 cktName2 
   >>>>>>>>>>---------------Till Here -------------<<<<<<<<<<*/

   s2iSpiceData spiceData = new s2iSpiceData(args[0]);
   try
   {
      spiceData.fillData();
   }
   catch (java.io.FileNotFoundException fnfe)
   {
      System.out.println(fnfe);
   }
   catch (java.io.IOException ioe)
   {
      System.out.println(ioe);
   }


   /* ** PRINT THE DATA THAT HAS BEEN FILLED FROM THE SPICE
   ***/
   Enumeration elems;
   String elemStr;

   elems = spiceData.getVoltElem();
   System.out.println("Voltage Elements");
   System.out.println("================");
   while (elems.hasMoreElements())
   {
      elemStr = (String) elems.nextElement(); 
      System.out.println(elemStr);
   }
   System.out.println();
   
   elems = spiceData.getCurrElem();
   System.out.println("Current Elements");
   System.out.println("================");
   while (elems.hasMoreElements())
   {
      elemStr = (String) elems.nextElement(); 
      System.out.println(elemStr);
   }
   System.out.println();
   
   elems = spiceData.getResElem();
   System.out.println("Resistor Elements");
   System.out.println("==================");
   while (elems.hasMoreElements())
   {
      elemStr = (String) elems.nextElement(); 
      System.out.println(elemStr);
   }
   System.out.println();
   
   elems = spiceData.getSubCktElem();
   System.out.println("Sub-circuits");
   System.out.println("==============");
   while (elems.hasMoreElements())
   {
      elemStr = (String) elems.nextElement(); 
      System.out.println(elemStr);
   }
   System.out.println();
   
   elems = spiceData.getSubCktInstantiated();
   System.out.println("SubCkt Names");
   System.out.println("==============");
   while (elems.hasMoreElements())
   {
      elemStr = (String) elems.nextElement(); 
      System.out.println(elemStr);
   }
   System.out.println();
   
   elems = spiceData.getNode();
   System.out.println("Nodes");
   System.out.println("=====");
   while (elems.hasMoreElements())
   {
      elemStr = (String) elems.nextElement(); 
      System.out.println(elemStr);
   }
   System.out.println();
   

   // A UNIQUE NODE NAME
   System.out.println("A unique node name is " + spiceData.getUniqueNodeName());
 }

}
